
String parseAsUrlToMap := method(
    m := Map clone
    s := self
    
    protocol := s before("://")
    s := s after("://")
    if (s == Nil, return)
    host := s before("/")
    if (host contains(":")) then (
      port := host after(":")
      host := host  before(":")
    ) else (
      port := "80"
    )

    request := s after("/")

    m atPut("protocol", protocol)
    m atPut("host", host)
    m atPut("port", port)
    m atPut("request", request)
    
    m
)

URL := Object clone do(
    url := ""
    setURL := method(s, 
	url = s asString
	parse
	self
    )

    clear := method(
	self protocol := Nil
	self host  := Nil
	self port  := 80
	self path  := "/"
	self query := Nil
	self error := Nil
    )

    escapeString := method(u,
      u := u clone asBuffer 
      u replace("%","%25")
      escapeCodes foreach(k, v, u replace(k, v))
      u asString
    )

    unescapeString := method(u,
      u := u clone asBuffer 
      escapeCodes foreach(k, v, u replace(v, k))
      u replace("%25","%")
      u asString
    )

    escapeCodes := method(
	c := Map clone
        c atPut(" ", "%20")
        c atPut("<","%3C")
        c atPut(">","%3E")
        c atPut("#","%23")
        //c atPut("%","%25")
        c atPut("{","%7B")
        c atPut("}","%7D")
        c atPut("|","%7C")
        //c atPut("backslash","%5C")
        c atPut("^","%5E")
        c atPut("~","%7E")
        c atPut("[","%5B")
        c atPut("]","%5D")
        c atPut("\`","%60")
        c atPut(";","%3B")
        c atPut("/","%2F")
        c atPut("?","%3F")
        c atPut(":","%3A")
        c atPut("@","%40")
        c atPut("=","%3D")
        c atPut("&","%26")
        c atPut("$","%24")
        c
    )

    referer := Nil
    setReferer := method(s, referer := s asString; self)

    parse := method(
	clear
	m := self url parseAsUrlToMap 
	
	if (m at("request")) then (
	  m atPut("request", "/" .. m at("request"))
	) else (
	  m atPut("request", "/")
	)
	m atPut("path", m at("request"))
	m foreach(k, v, self setSlot(k, v); /*write(k, "=", v, "\n")*/)
	if (port) then (port = port asNumber) else (port = 80)
    )

    fetch := method(
	if (protocol == "http", return self fetchHttp)
	Exception raise("URL.fetch", "protocol '" .. protocol .. "' unsupported")
    )

    stopFetch := method(socket close)

    requestHeader := method(
	header := Buffer clone
	//write("request = [", self request, "]\n")
	header append("GET ", self request," HTTP/1.0\r\n")
	header append("User-Agent: Mozilla/4.0\r\n")
	header append("Host: ", host, "\r\n")
	if (referer, header append("Referer: ", referer, "\r\n"))
	header append("Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png, image/tiff, multipart/x-mixed-replace, */*\r\n")
	header append("Accept-Charset: iso-8859-1, utf-8, iso-10646-ucs-2, macintosh, windows-1252, *\r\n")
	header append("Accept-Language: en, *;q=0.5\r\n")
	header append("\r\n")
	header
    )

    fetchHttp := method(
	ip := Host clone setName(host) address
	socket := Socket clone setHost(ip) setPort(port) 
	socket connect 
	if (socket error, error := socket error; return Nil) 

	socket write(self requestHeader)
	while(socket read, Nop)
	if (socket error) then (write(socket error, "\n"); return ) 
	/*write("read ", socket readBuffer size, " bytes\n")*/
	socket close

	b := socket readBuffer
	/*write("buffer[", b, "]\n")*/
	i := b find("\r\n\r\n")
	if (i, b removeSlice(0, i+4); return b)
	i := b find("\n\n");
	if (i, b removeSlice(0, i+2); return b)
	b
    )

    fetchToFile := method(file,
	ip := Host clone setName(host) address
	socket := Socket clone setHost(ip) setPort(self port) 
	socket connect 

	if (socket error, error := socket error; return Nil) 

	socket write(self requestHeader)
	b := socket readBuffer
	while(socket read, 
	  if (b contains("\r\n\r\n") or b contains("\n\n"), break)
	)
	if (socket error) then ( write(socket error, "\n"); return )

	if (b clipBeforeEndOf("\r\n\r\n") == Nil, b clipBeforeEndOf("\n\n"))
	file write(b)
	b empty

	while(socket read, 
	  file write(b)
	  b empty
	)
	socket close
	if (socket error) then ( write(socket error, "\n"); return )     
    )


    childUrl := method(u, 
	if (u beginsWith("http") == Nil, 
	  u = Path with(self url cloneRemoveLastPathComponent, u)
	)
	URL clone setURL(u) setReferer(self url)
    )
)


