/*   
copyright: Steve Dekorte, 2004. All rights reserved.
license: See _BSDLicense.txt.

object: SGMLTag
prototype: Object

module: IoServer
description: Representation of an SGML / HTML / XML tag. 
*/

/*
method: setName(aString)
description: Sets the tag name. Returns self.
*/
/*
method: name
description: Returns the tag name
*/

/*
method: attributes
description: Returns a Map containing the tag's attributes.
*/

/*
method: subitems
description: Returns a List containing the tag's subitems.
*/

/*
method: asString
description: Returns a String representation of the tag and all of it's subitems.
*/

Sequence writeToStream := method(b, b appendSeq(self))

SGMLTag := Object clone do(
    type := "SGMLTag"
    
    newSlot("name", Nil)
    
    init := method(
	self attributes := Map clone
	self subitems := List clone
    )

    
    asString := method( 
	b := Buffer clone
	writeToStream(b)
	b asString
    )
    
    writeToStream := method(b,
	if (name,
	    b appendSeq("<", name)
	    attributes keys sort foreach(k,
		v := attributes at(k) 
		b appendSeq(" ", k, "=\"", v, "\"")
	    )
	    b appendSeq(">")
	)
	subitems foreach(item, item writeToStream(b))
	if (name, b appendSeq("</", name, ">"))
    )
)

SGMLParser do(
    type := "SGMLParser"
    init := method(
	self stack := List clone
	self rootTag := SGMLTag clone
	stack push(rootTag)
    )
    
    top := method(
	if (stack last, stack last, rootTag)
    )
    
    startElement := method(name,
	tag := SGMLTag clone setName(name)
	top subitems append(tag)
	stack push(tag)
    )

    endElement := method(name,
	stack pop
    )
    
    newAttribute := method(k, v,
	top attributes atPut(k, v)
    )

    newText := method(name,
	top subitems append(name)
    )
    
    tagForString := method(aString,
	parse(aString)
	rootTag
    )
)

m := message(
	asHTML := method(SGMLParser clone tagForString(self))
	asXML  := method(SGMLParser clone tagForString(self))
	asSGML := method(SGMLParser clone tagForString(self))
)

Sequence doMessage(m)
