/*
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
*/

#include "Descriptor.h"
#include "SocketManager.h"

Descriptor *Descriptor_new(void)
{
    Descriptor *self = (Descriptor *)calloc(1, sizeof(Descriptor));
    Descriptor_error_(self, "");
    self->fd = -1;
    self->ev = calloc(1, sizeof(struct event));
    self->timeout = calloc(1, sizeof(struct timeval));
    self->timeout->tv_sec = 10 * 356 * 60 * 60;
    return self;
}

void Descriptor_free(Descriptor *self)
{
    Descriptor_close(self);
    free(self->error);
    free(self->ev);
    free(self->timeout);
    free(self);
}

void Descriptor_socketManager_(Descriptor *self, void *s) 
{ 
    self->socketManager = s; 
}

void *Descriptor_socketManager(Descriptor *self) 
{ 
    return self->socketManager; 
}

void Descriptor_scheduler_(Descriptor *self, Scheduler *s) 
{ 
    self->scheduler = s; 
}

Scheduler *Descriptor_scheduler(Descriptor *self) 
{ 
    return self->scheduler; 
}

void Descriptor_pause(Descriptor *self) 
{
    self->coroId = Scheduler_currentCoro(self->scheduler);
    Coro_check(self->coroId);
    Coro_pause(self->coroId);
}

void Descriptor_resume(Descriptor *self)
{ 
    Coro_resume(self->coroId);
}

/*
void Descriptor_errorCallback_(Descriptor *self, DescriptorErrorCallback *f) { 
{
    self->errorCallback = f; 
}
*/

void Descriptor_removeFromSocketManager(Descriptor *self) 
{ 
    self->socketManager = 0x0; 
}

void Descriptor_error_(Descriptor *self, const char *e)
{
    self->error = strcpy((char *)realloc(self->error, strlen(e)+1), e); 
}

char *Descriptor_error(Descriptor *self) 
{ 
    return self->error; 
}

int Descriptor_hasError(Descriptor *self) 
{ 
    return (self->error && strlen(self->error)) ? 1 : 0; 
}

int Descriptor_isOpen(Descriptor *self) 
{ 
    return self->fd > -1 ? 1 : 0; 
}

int Descriptor_waitOnReadOrTimeout_(Descriptor *self, double timeout)
{
    self->timeout->tv_sec = timeout;
    SocketManager_addReadEntry_(self->socketManager, self);
    return (self->fd > -1) ? 0 : -1; /* -1 for error */
}

int Descriptor_waitOnWriteOrTimeout_(Descriptor *self, double timeout)
{
    self->timeout->tv_sec = timeout;
    SocketManager_addWriteEntry_(self->socketManager, self);
    return (self->fd > -1) ? 0 : -1; /* -1 for error */
}

void Descriptor_close(Descriptor *self)
{
    if (self->fd > -1) 
    {
	if (self->socketManager && Descriptor_hasManagerEntry(self))
	{ 
	    SocketManager_removeEntry_(self->socketManager, self); 
	}
	
	PortableSocket_close(self->fd);
	self->fd = -1;
    }
}

int Descriptor_hasManagerEntry(Descriptor *self) 
{ 
    return self->hasManagerReadEntry || self->hasManagerWriteEntry; 
}

void Descriptor_hasManagerReadEntry_(Descriptor *self, int i) 
{ 
    self->hasManagerReadEntry = i; 
}

void Descriptor_hasManagerWriteEntry_(Descriptor *self, int i) 
{ 
    self->hasManagerWriteEntry = i; 
}




