/*
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
*/

#ifndef DESCRIPTOR_DEFINED
#define DESCRIPTOR_DEFINED 1

#include "PortableSocket.h"
#include <time.h>
#include "event.h"
#include "Scheduler.h"

typedef void (DescriptorErrorCallback)(void *, const char *);
typedef void (DescriptorRemoveCallback)(void *);
  
typedef struct 
{
  void *socketManager; 
  SOCKET_DESCRIPTOR fd; 
  struct timeval *timeout;

  void *scheduler;
  Coro *coroId;
  
  char *error; 
  struct event *ev; 
  int hasManagerReadEntry;
  int hasManagerWriteEntry;
} Descriptor;

Descriptor *Descriptor_new(void);
void Descriptor_free(Descriptor *self);

void Descriptor_socketManager_(Descriptor *self, void *s);
void *Descriptor_socketManager(Descriptor *self);

void Descriptor_nextTimeout_(Descriptor *self, time_t v);
time_t Descriptor_nextTimeout(Descriptor *self);

/* -- Scheduler -- */
void Descriptor_scheduler_(Descriptor *self, Scheduler *s);
Scheduler *Descriptor_scheduler(Descriptor *self);

void Descriptor_pause(Descriptor *self);
void Descriptor_resume(Descriptor *self);

/* -- SocketManager -- */

/*
void Descriptor_errorCallback_(Descriptor *self, DescriptorErrorCallback *f);
*/
void Descriptor_removeFromSocketManager(Descriptor *self);

void Descriptor_error_(Descriptor *self, const char *e);
char *Descriptor_error(Descriptor *self);

int Descriptor_hasError(Descriptor *self);

int Descriptor_isOpen(Descriptor *self);

int Descriptor_waitOnReadOrTimeout_(Descriptor *self, double timeout);
int Descriptor_waitOnWriteOrTimeout_(Descriptor *self, double timeout);
void Descriptor_close(Descriptor *self);

int Descriptor_hasManagerEntry(Descriptor *self);
void Descriptor_hasManagerReadEntry_(Descriptor *self, int i);
void Descriptor_hasManagerWriteEntry_(Descriptor *self, int i);

#endif
