/*
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docDescription("Notes: Need to move the timeout handling into the SocketManager so we can avoid polling.")
*/

#ifndef SOCKET_DEFINED
#define SOCKET_DEFINED 1

#include "SocketManager.h"
#include "ByteArray.h"

typedef struct Socket
{
  Descriptor *descriptor;

  int socketType;
  struct sockaddr_in toAddress;   /* only needed for connect */
  /*struct hostent *hent;*/       /* only needed for connect */
  struct sockaddr_in fromAddress; /* only needed for udp receive */
  
  time_t connectTime;
  
  double connectTimeout;
  double readTimeout;
  double writeTimeout;
  
  char *host;
  unsigned int port;
  
  ByteArray *buffer;
  size_t readSize;
  
  unsigned int ownsBuffer : 1;
  unsigned int unused : 7;
} Socket;

Socket *Socket_new(void);
void Socket_free(Socket *self);

void Socket_externalBuffer_(Socket *self, ByteArray *ba);

void Socket_fromAddress_(Socket *self, struct sockaddr *address);

char *Socket_error(Socket *self);
int Socket_hasError(Socket *self);

void Socket_host_(Socket *self, const char *host);
char *Socket_host(Socket *self);

void Socket_port_(Socket *self, unsigned int port);
unsigned int Socket_port(Socket *self);

void Socket_readSize_(Socket *self, int t);
int Socket_readSize(Socket *self);

void Socket_connectTimeout_(Socket *self, time_t timeout);
time_t Socket_connectTimeout(Socket *self);

void Socket_readTimeout_(Socket *self, time_t timeout);
time_t Socket_readTimeout(Socket *self);

int Socket_isOpen(Socket *self);

void Socket_isDatagram_(Socket *self, int b);
int Socket_isDatagram(Socket *self);

void Socket_open(Socket *self);
int Socket_openFifo_(Socket *self, const char *path);
int Socket_connect(Socket *self);

size_t Socket_read(Socket *self);
int Socket_justRead(Socket *self);
int Socket_write(Socket *self, unsigned char *data, int dataLength);
size_t Socket_activeBytesPerSecond(Socket *self);
int Socket_timeoutExceeded(Socket *self);
void Socket_close(Socket *self);

ByteArray *Socket_buffer(Socket *self);

size_t Socket_udpWrite(Socket *self, unsigned char *data, int length);
size_t Socket_udpRead(Socket *self);
char *Socket_udpFromAddressIp(Socket *self);

/* --- Server ---------- */

void Socket_serverOpen(Socket *self);
Socket *Socket_acceptNewSocket(Socket *self);

#endif
