/*
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docDescription("Selects on sockets and checks for timeouts.")
*/

#ifndef SOCKETMANAGER_DEFINED
#define SOCKETMANAGER_DEFINED 1

/*
#undef FD_SETSIZE
#define FD_SETSIZE 36000
*/

#include "PortableSocket.h"
#include "Hash.h"
#include "Scheduler.h"
#include "Descriptor.h"

#include "event.h"

/*
#define LIBEVENT_MULTISTATE 1
*/
#undef LIBEVENT_MULTISTATE

typedef void (SocketManagerCallback)(void *);

typedef struct SocketManager
{
    Hash *descriptors;

    #ifdef LIBEVENT_MULTISTATE
    struct eventctxt *ec;
    #endif

    //struct timeval timeout;

    void *target;
    SocketManagerCallback *callback; 

    Scheduler *scheduler;
    unsigned int timeoutMicroseconds;
} SocketManager;

SocketManager *SocketManager_new(void);
void SocketManager_free(SocketManager *self);

void SocketManager_scheduler_(SocketManager *self, Scheduler *s);
Scheduler *SocketManager_scheduler(SocketManager *self);

void SocketManager_timeoutSeconds_(SocketManager *self, float v);
float SocketManager_timeoutSeconds(SocketManager *self);

void SocketManager_target_(SocketManager *self, void *target);
void SocketManager_callback_(SocketManager *self, SocketManagerCallback *callback);

/* ------------------------------------- */
int SocketManager_descriptorCount(SocketManager *self);

void SocketManager_addReadEntry_(SocketManager *self, Descriptor *entry);
void SocketManager_addWriteEntry_(SocketManager *self, Descriptor *entry);
void SocketManager_removeEntry_(SocketManager *self, Descriptor *entry);

void SocketManager_timer(SocketManager *self);

long setDescriptorLimitToMax(void);
long SocketManager_descriptorLimit(SocketManager *self);
long SocketManager_descriptorLimitMax(SocketManager *self);
long SocketManager_setDescriptorLimit_(SocketManager *self, long n);

#endif
