/*
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
*/

#define IoCatch_C
#include "IoCatch.h"
#undef IoCatch_C
#include "IoState.h"
#include "IoNil.h"
#include "Scheduler.h"


IoCatch *IoCatch_new(void *state)
{
  IoCatch *self = calloc(1, sizeof(IoCatch));
  self->savedStack = calloc(1, sizeof(jmp_buf));
  /*self->savedStack = calloc(1, sizeof(jmp_buf) + CSTACK_STACKSIZE);*/
  self->state = state;
  return self;
}

/* ------------------------------------------------------------------- */

void IoCatch_free(IoCatch *self)
{
  free(self->savedStack);
  free(self);
}

/* -------------------------------------------------------------- */

void IoCatch_push(IoCatch *self)
{
  IoState *state = self->state;
  IoCoroutine *coro = state->currentActor->coro;
  self->previous = IoCoroutine_exceptionCatch(coro);
  /*printf("pushing %p on top of %p in coro %p\n", (void *)self, (void *)self->previous, (void *)coro);*/
  IoCoroutine_exceptionCatch_(coro, self);
  self->stackMark = Stack_pushMarkPoint(coro->retainedValues);
}

void IoCatch_pop(IoCatch *self)
{
  IoState *state = self->state;
  IoCoroutine *coro = state->currentActor->coro;
  /*printf("poping back to %p in coro %p\n", (void *)self->previous, (void *)coro);*/
  IoCoroutine_exceptionCatch_(coro, self->previous);
  self->previous = 0x0;
  Stack_popToMark_(coro->retainedValues, self->stackMark);
}

IoObject *IoCatch_tryFunc_(IoCatch *self, IoCatchCallback *func, 
  void *arg1, void *arg2, void *arg3, IoException **e)
{
  IoObject *result;
  IoCatch_push(self);
  
  if (!CSTACK_SETJMP(*(self->savedStack)))
  {
    result = (*func)(arg1, arg2, arg3);
    IoCatch_pop(self);
    *e = 0x0;
  }
  else
  {
    IoState *state = self->state;
    *e = state->currentActor->coro->exception;
    IoCatch_pop(self);
    result = state->ioNil;
  }
  return result;
}

void IoCatch_longJump(IoCatch *self)
{
  /*IoState *state = self->state;
  Coro *coro = state->currentActor->coro->cid;
  jmp_buf *stack;
  if (!coro) 
  {
    coro = Coro_new();
    state->currentActor->coro->cid = coro; 
  }
  stack = coro->env;
  coro->env = self->savedStack;
  self->savedStack = stack;
  CSTACK_LONGJMP(*(coro->env), 1);
  */
  CSTACK_LONGJMP(*(self->savedStack), 1);
}

