/*
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
*/

#include "IoCoroutine.h"
#include "IoState.h"
#include "IoObject.h"

IoCoroutine *IoCoroutine_new(void)
{
    IoCoroutine *self = (IoCoroutine *)calloc(1, sizeof(IoCoroutine));
    self->queue = Queue_new();
    /* create base catch? */
    self->retainedValues = Stack_new();
    return self;
}

void IoCoroutine_free(IoCoroutine *self)
{
    if (self->queue) 
    {
	Queue_free(self->queue);
    }
    
    Stack_free(self->retainedValues);
    free(self);
}

void IoCoroutine_mark(IoCoroutine *self)
{ 
    Queue_do_(self->queue, (QueueDoCallback *)IoObject_makeGrayIfWhite); 
    /*printf("Coro %p marking stack\n", (void *)self);*/
    Stack_do_(self->retainedValues, (ListDoCallback *)IoObject_makeGrayIfWhite);
    
    if (self->exception) 
    {
	IoObject_makeGrayIfWhite(self->exception);
    }
}

size_t IoCoroutine_memorySize(IoCoroutine *self)
{
    size_t t = sizeof(IoCoroutine);
    
    t += Queue_memorySize(self->queue);
    /*
     Stack *s = Stack_newCopyWithNullMarks(self->exceptionCatchStack);
     IoExceptionCatch *ec = Stack_pop(s);
     while (ec)
     {
	 count += IoExceptionCatch_memorySize(self->retainedValues);
	 ec = Stack_pop(s);
     }
     Stack_free(s);
     */
    return t;
}

void IoCoroutine_compact(IoCoroutine *self)
{
    Queue_compact(self->queue);
}

int IoCoroutine_retainingCount(IoCoroutine *self)
{ 
    return Stack_totalSize(self->retainedValues); 
}

void IoCoroutine_show(IoCoroutine *self)
{
    Stack_do_(self->retainedValues, (StackDoCallback *)IoObject_show);
    printf("\n");
}

