/*#io
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
*/

#ifndef IOEXCEPTION_DEFINED
#define IOEXCEPTION_DEFINED 1

#include "Common.h"
#include "Stack.h"
#include "IoSeq.h"
#include "Scheduler.h"

#ifdef __cplusplus
extern "C" {
#endif

#define ISEXCEPTION(self) \
  IoObject_hasCloneFunc_(self, (TagCloneFunc *)IoException_rawClone)
  
typedef IoObject IoException;

typedef struct
{
    Stack *retainStack; /* retainStack of coro when raised */
    jmp_buf *savedStack;
    unsigned char isResumable;
} IoExceptionData;

IoException *IoException_proto(void *state);
IoException *IoException_rawClone(IoException *self);
IoException *IoException_new(void *state);
IoException *IoException_newWithDescription_(void *state, IoSymbol *d);

void IoException_name_(IoException *self, IoSymbol *s);
IoSymbol *IoException_name(IoException *self);

void IoException_description_(IoException *self, IoSymbol *s);
IoSymbol *IoException_description(IoException *self);

void IoException_caughtMessage_(IoException *self, IoMessage *m);
IoMessage *IoException_caughtMessage(IoException *self);

void IoException_free(IoException *self);
void IoException_mark(IoException *self);
void IoException_show(IoException *self);

/* ------------------------------------------------- */
void IoException_justRaise(IoObject *self);
IoObject *IoException_raise(IoObject *self, IoObject *locals, IoMessage *m);
IoObject *IoException_raiseResumable(IoObject *self, IoObject *locals, IoMessage *m);
IoObject *IoException_catch(IoObject *self, IoObject *locals, IoMessage *m);

void IoException_rawPass(IoObject *self);
IoObject *IoException_pass(IoObject *self, IoObject *locals, IoMessage *m);
IoObject *IoException_resume(IoObject *self, IoObject *locals, IoMessage *m);
IoObject *IoException_isResumable(IoObject *self, IoObject *locals, IoMessage *m);

/* --- Stack trace ---------------------------------------------- */
void IoException_printBackTrace(IoException *self);
IoObject *IoException_callStack(IoObject *self);
IoObject *IoException_backTraceString(IoObject *self, IoObject *locals, IoMessage *m);

#ifdef __cplusplus
}
#endif
#endif
