/*   
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
*/

#ifdef IOOBJECTGROUP_C 
#define IO_IN_C_FILE
#endif
#include "Common_inline.h"
#ifdef IO_DECLARE_INLINES

IOINLINE void IoObjectGroup_isRemoving_(IoObjectGroup *self, IoObject *v)
{ 
    if (self->first == v) 
    {
	self->first = (IoObject *)(v->mark.next); 
    }
    
    if (self->last == v)  
    {
	self->last  = (IoObject *)(v->mark.previous); 
    }
    
    /* unlink it */
    if (v->mark.previous) 
    { 
	((IoObject *)(v->mark.previous))->mark.next = v->mark.next; 
    }
    
    if (v->mark.next)
    { 
	((IoObject *)(v->mark.next))->mark.previous = v->mark.previous; 
    }
    
    v->mark.next = 0x0;
}

#define IOOBJECTGROUP_APPEND_FIRST 1

#ifdef IOOBJECTGROUP_APPEND_FIRST

IOINLINE void IoObjectGroup_append_(IoObjectGroup *self, IoObject *v)
{
    /* append it to the beginning */
    v->mark.previous = 0x0;
    v->mark.next = self->first;
    
    if (self->first)
    {
	self->first->mark.previous = v;
    }
    
    self->first = v;
    
    if (!self->last) 
    {
	self->last = v;
    }
    
    v->mark.color = self->mark.color;
}

#else

IOINLINE void IoObjectGroup_append_(IoObjectGroup *self, IoObject *v)
{
    // append it to the end 
    v->mark.previous = self->last;
    v->mark.next = 0;
    
    if (self->last) 
    {
	self->last->mark.next = v;
    }
    
    self->last = v;

    if (!self->first) 
    {
	self->first = v;
    }
    
    v->mark.color = self->mark.color;
}

#endif

IOINLINE void IoObjectGroup_appendGroup_(IoObjectGroup *self, IoObjectGroup *other)
{
    /* append a whole group to this one */
    if (other->first)
    {
	other->first->mark.previous = self->last;
    }
    
    if (self->last) 
    {
	self->last->mark.next = other->first;
    }
    
    self->last = other->last;

    if (!self->first) 
    {
	self->first = other->first;
    }
}

#undef IO_IN_C_FILE
#endif
