/*
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
*/

#include "IoState.h"

#include "IoObject.h"
#include "IoObject_actor.h"
#include "IoNil.h"
#include "IoSeq.h"
#include "IoNumber.h"
#include "IoWeakLink.h"

#include <time.h>
#include <limits.h> 
#include <stdlib.h>

void IoState_updateCachedCollectorColors(IoState *self)
{
    self->currentBlackColor = self->blackGroup->mark.color;
    self->currentWhiteColor = self->whiteGroup->mark.color;
}

IoObject *IoState_retain_(IoState *self, IoObject *v)
{ 
    /*IoObject_makeGrayIfWhite(v);*/
    /*IoObject_createSlotsIfNeeded(v);*/
    return (IoObject *)List_append_(self->retainedValues, v); 
}

void IoState_release_(IoState *self, IoObject *v)
{ 
    List_removeLast_(self->retainedValues, v); 
}

void IoState_gcInitPhase(IoState *self)
{
    /* assume all values currently white */
    if (self->lobby)
    {
	IoObject_makeGrayIfWhite(self->lobby);
	IoObject_mark(self->lobby);
    }
    /*List_do_(self->actors, (ListDoCallback *)IoObject_makeGrayIfWhite); */
}

int IoState_gcSweepPhase(IoState *self)
{
    int count;
    int debug = self->debugCollector;
    
    if (debug) 
    {
	fputs("--- collector start ---\n", stdout);
    }
    
    /* mark the permanentally retained values */
    /*if (debug)  printf("retainedValues: %i\n", List_size(self->retainedValues));*/
    List_do_(self->retainedValues, (ListDoCallback *)IoObject_makeGrayIfWhite);
    
    /* Marking the actors will mark their stacks. */
    /* printf("State marking %i actors\n", List_size(self->actors)); */
    /*if (debug)  printf("actors: %i\n", List_size(self->actors));*/
    List_do_(self->actors, (ListDoCallback *)IoObject_markActor);
    /*if (debug)  printf("primitives: %i\n", PHash_count(self->primitiveProtos));*/
    
    {
	IoSymbol *k = Hash_firstKey(self->primitiveProtos);
	
	while (k)
	{
	    IoObject *v = Hash_at_(self->primitiveProtos, k);
	    IoObject_makeGrayIfWhite(v);
	    k = Hash_nextKey(self->primitiveProtos);
	}
    }
    
    /*PHash_do_(self->primitiveProtos, (PHashDoCallback *)IoObject_makeGrayIfWhite);*/
    
    if (debug)
    {
	printf("white: %i\n", IoObjectGroup_count(self->whiteGroup));
	printf("gray:  %i\n", IoObjectGroup_count(self->grayGroup));
	printf("black: %i\n", IoObjectGroup_count(self->blackGroup));
	printf("recycled: %i\n", List_size(self->recycledObjects));
	/*
	 if(IoObjectGroup_sharesWithGroup_(self->whiteGroup, self->grayGroup))
	 { printf("IoObjectGroup_sharesWithGroup_(self->whiteGroup, self->grayGroup)\n"); }
	 
	 if(IoObjectGroup_sharesWithGroup_(self->whiteGroup, self->blackGroup))
	 { printf("IoObjectGroup_sharesWithGroup_(self->whiteGroup, self->blackGroup)\n"); }
	 
	 if(IoObjectGroup_sharesWithGroup_(self->grayGroup, self->blackGroup))
	 { printf("IoObjectGroup_sharesWithGroup_(self->grayGroup, self->blackGroup)\n"); }
	 */
    }
    
    while (!IoObjectGroup_isEmpty(self->grayGroup)) 
    {
	IoObjectGroup_markValues(self->grayGroup);
    }
    
    if (debug) 
    {
	printf("left:  %i\n", IoObjectGroup_count(self->blackGroup));
    }
    
    /* move white items into free group */
    IoObject_moveToGroup_(self->whiteGroup, self->freeGroup);
    
    /* dynamically adjust wait until sweep size */
    /*
    self->lastReferencedObjectCount = IoObjectGroup_count(self->blackGroup);
    self->allocsPerSweep  = 
    self->lastReferencedObjectCount > STATE_DEFAULT_COLECT_AFTER ? 
    self->lastReferencedObjectCount : STATE_DEFAULT_COLECT_AFTER;
    */
    /*printf("self->allocsPerSweep = %i\n", self->allocsPerSweep);*/
    
    /* swap black group with the now empty white group */
    
    self->sweepsSinceLastGeneration ++;
    
    if (!self->sweepsPerGeneration ||
	self->sweepsSinceLastGeneration == self->sweepsPerGeneration)
    {
	IoObjectGroup *vg = self->blackGroup;
	self->blackGroup = self->whiteGroup;
	self->whiteGroup = vg;
	self->sweepsSinceLastGeneration = 0;
	IoState_updateCachedCollectorColors(self);
    }
    
    count = IoObjectGroup_freeValues(self->freeGroup);
    
    IoState_gcInitPhase(self);
    self->newValueSinceSweepCount = 0;
    
    if (debug)
    {
	/*printf("generation: %i\n", self->sweepsSinceLastGeneration);*/
	printf("freed: %i\n", (int)count);
	printf("perm:  %i\n", (int)List_size(self->retainedValues));
	printf("actors: %i\n", (int)List_size(self->actors));
	printf("protos: %i\n", (int)Hash_count(self->primitiveProtos));
	printf("unique strings: %i\n", (int)SkipDB_count(self->symbols));
	
	{
	    int i, max = List_size(self->actors);
	    
	    for (i = 0; i< max; i++) 
	    {
		IoObject *actor = List_at_(self->actors, i);
		/*if (Queue_length(actor->coro->queue))*/ 
		{ IoObject_printCoroInfo(actor); }
	    }
	}
	
	fputs("--- collector done  ---\n", stdout);
    }

    return count;
}

int IoState_collectGarbage(IoState *self) 
{ 
    return IoState_gcSweepPhase(self); 
}

void IoState_makeBlackFromGray_(IoState *self, IoObject *v)
{ 
    IoObjectGroup_isRemoving_(self->grayGroup, v);
    IoObjectGroup_append_(self->blackGroup, v); 
}

void IoState_makeGrayFromWhite_(IoState *self, IoObject *v)
{ 
    IoObjectGroup_isRemoving_(self->whiteGroup, v);
    IoObjectGroup_append_(self->grayGroup, v); 
}

void IoState_makeGrayFromBlack_(IoState *self, IoObject *v)
{ 
    IoObjectGroup_isRemoving_(self->blackGroup, v);
    IoObjectGroup_append_(self->grayGroup, v); 
}

void IoState_immediatelyFreeIfUnreferenced_(IoState *self, IoObject *v)
{
    #ifdef IO_BLOCK_RECYCLING
    if (0x0 == IoObject_isReferenced(v))
    { 
	IoState_immediatelyFree_(self, v); 
    }
    #endif
}

void IoState_immediatelyFree_(IoState *self, IoObject *v)
{ 
    unsigned int color = v->mark.color;
    
    if (color == self->whiteGroup->mark.color)
    { 
	IoObjectGroup_isRemoving_(self->whiteGroup, v); 
	/*IoObjectGroup_append_(self->freeGroup, v);*/
    }
    else if (color == self->grayGroup->mark.color)
    { 
	IoObjectGroup_isRemoving_(self->grayGroup, v); 
    }
    else if (color == self->blackGroup->mark.color)
    { 
	IoObjectGroup_isRemoving_(self->blackGroup, v); 
    }
    
    IoObject_free(v);
    
    /*if (self->newValueCount) */
    self->newValueCount --;
    /*printf("unref newValueCount = %i\n", self->newValueCount);*/
}

/*
 void IoState_immediatelyFreeIfUnreferenced_(IoState *self, IoObject *v)
 { 
     if (0 == v->mark.refCount) IoState_immediatelyFree_(self, v);
 }
 */

void IoState_gcMarkPhase(IoState *self)
{
    IoObjectGroup_markValues(self->grayGroup);
    
    self->newValueSinceSweepCount += self->newValueCount;
    self->newValueCount = 0;
    
    if ( /*IoObjectGroup_isEmpty(self->grayGroup) ||*/
	self->newValueSinceSweepCount > self->allocsPerSweep) 
    {
	if (Stack_count(self->currentRetainStack) > 40000)
	{ 
	    IoState_error_description_(self, 0x0, "IoMessage", 
		"stack overflow (about %i pointers)", 
		Stack_count(self->currentRetainStack)/sizeof(char *)); 
	}
	
	IoState_gcSweepPhase(self);
	self->newValueSinceSweepCount = 0;
    }
}

int IoState_hasValue_(IoState *self, IoObject *v)
{ 
    return 
	IoObjectGroup_hasValue_(self->whiteGroup, v) || 
	IoObjectGroup_hasValue_(self->grayGroup,  v) ||
	IoObjectGroup_hasValue_(self->blackGroup, v);
}

int IoState_garbageCollectorIsPaused(IoState *self)
{ 
    return (self->collectorPauseCount != 0); 
}

void IoState_pushCollectorPause(IoState *self)
{
    self->collectorPauseCount ++;
}

void IoState_popCollectorPause(IoState *self)
{
    self->collectorPauseCount --;
    
    if ((self->collectorPauseCount == 0) && 
        (self->newValueCount > self->newValueCountToUpdate)) 
    {
	IoState_gcMarkPhase(self);
    }
    else if (self->collectorPauseCount < 0)
    {
	IoState_error_description_(self, 0x0, "IoVM", 
	    "negative collector pause count");     
    }
}

IoObject *IoState_addValue_(IoState *self, IoObject *v)
{ 
    /*
    if (!(v->tag))
    {
	fputs("IoState_addValue_() Error: missing tag\n", stdout);
	exit(1);
    }
    */
    
    //printf("IoState_addValue_(%p)\n", (void *)v);
    /*printf("newValueCount = %i\n", self->newValueCount);*/

    /*
    if ((self->collectorPauseCount == 0) && 
	(self->newValueCount > self->newValueCountToUpdate)) 
    {
	IoState_gcMarkPhase(self);
    }
    */
    /*
    if (IoState_hasValue_(self, v))
    {
	printf("error - value added more than once to IoState\n");
	exit(-1);
    }
    */
    
    if (!v->mark.knownToCollector)
    {
	v->mark.knownToCollector = 1;
	IoObjectGroup_append_(self->whiteGroup, v);
	self->newValueCount ++;
    }
    else
    {
	printf("object known to collector\n");
    }
    
    IoState_stackRetain_(self, v);
    /*printf("new %s\n", v->tag->name);*/
    return v;
}

/*--- weak links ---------------------------------------------*/

void IoState_addWeakLink_(IoState *self, IoWeakLink *w)
{
    IoObject *link = IoWeakLink_rawLink(w);
    IoWeakLink *first = Hash_at_(self->valueToWeakLink, link);
    
    if (first)
    {
	/* not the first, so add it to the chain for this value */
	IoWeakLink_appendWeakLink_(first, w);
    }
    else
    {
	/* it's the first, so set it in the hash */
	/*printf("adding value %p with weaklink %p\n", link, w);*/
	Hash_at_put_(self->valueToWeakLink, link, w);
    }
}

void IoState_removeWeakLink_(IoState *self, IoWeakLink *w)
{
    /* the WeakLink will unlink itself, we just need to check if it's the first */
    IoWeakLink *first = Hash_at_(self->valueToWeakLink, IoWeakLink_rawLink(w));
    
    if (first == w)
    {
	IoObject *k = IoWeakLink_rawLink(first);
	IoObject *v = IoWeakLink_rawNextWeakLink(first);
	Hash_at_put_(self->valueToWeakLink, k, v);
    }
}

void IoState_willFreeWeakLinkedValue_(IoState *self, IoObject *value)
{
    IoWeakLink *first = Hash_at_(self->valueToWeakLink, value);
    /*printf("collecting value %p with first weaklink %p\n", value, first);*/
    
    if (first) 
    {
	IoWeakLink_collected(first);
	/*Stack_push_(self->collectedWeakLinks, first);*/
	Hash_at_put_(self->valueToWeakLink, value, 0x0);
    }
}

/* --- persistence -------------------------------- */

void IoState_willFreePersistentValue_(IoState *self, IoObject *value)
{
    if (!self->isCleaningUp)
    { 
	IoStore_willFreePersistentObject_(self->store, value);
    }
}

/* ------------------------------------------------ */

size_t IoState_memorySize(IoState *self)
{ 
    size_t total = 0;
    /*
    total += IoObjectGroup_sum_(self->whiteGroup, IoObject_memorySize);
    total += IoObjectGroup_sum_(self->grayGroup,  IoObject_memorySize);
    total += IoObjectGroup_sum_(self->blackGroup, IoObject_memorySize);
     */
    return total;
}

void IoState_compact(IoState *self)
{ 
    /*
    IoObjectGroup_do_(self->whiteGroup, IoObject_compact);
    IoObjectGroup_do_(self->grayGroup,  IoObject_compact);
    IoObjectGroup_do_(self->blackGroup, IoObject_compact);
    */
}
