
Object do(
    docsSlot := method(
	self hasSlot("docs") ifNil(self docs := Object clone do(slots := Object clone))
	self docs
    )
    
    docSlot := method(k, v, 
	entry := Object clone 
	entry description := v asSymbol
	if (k containsSeq("("), 
	    entry args := k afterSeq("(") beforeSeq(")") split(",") map(v, v strip)
	)	
	self docsSlot slots setSlot(k beforeSeq("("), entry)

	//self docsSlot slots setSlot(k beforeSeq("("), v)
	//writeln("docSlot(", k, ", ", self docsSlot slots getSlot(k), ")")
    )
    
    docCopyright := method(v, 
	k := thisMessage name asMutable removePrefix("doc") makeFirstCharacterLowerCase
	self docsSlot setSlot(k, v)
    )
    
    docLicense := getSlot("docCopyright")
    docObject := getSlot("docCopyright")
    docDescription := getSlot("docCopyright")
    docCredits := getSlot("docCopyright")
    docInclude := getSlot("docCopyright")
    docDependsOn := getSlot("docCopyright")

    ioDoc := method(
	//docsSlot
	if(thisMessage argAt(0), self doMessage(thisMessage argAt(0)))
    )


    
    docSlot("eval(...)", "Performs the argument (in the same manner as do() ) and returns the result. ")
    eval := method(self doMessage(thisMessage argAt(0)))
    
    docSlot("list(...)", "Returns a List containing the arguments.")
    list := method(thisMessage argsEvaluatedIn(sender))
    
    docSlot("..(arg)", ".. is an alias for: method(arg, self asString append(arg asString))")
    setSlot("..", method(arg, self asString appendSeq(arg asString)))
    
    docSlot("newSlot(slotName, aValue)", 
    """Creates a getter and setter for the slot with the name slotName 
    and sets it's default the value aValue. Returns self. For example, 
    newSlot("foo", 1) would create slot named foo with the value 1 as well as a setter method setFoo().""")

    newSlot := method(name, value, 
	self setSlot(name, value)
	self setSlot("set" .. name asCapitalized, 
		doString("method(" .. name .. " = sender doMessage(thisMessage argAt(0)); self)"))
	value
    )
    
    docSlot("launchFile(pathString)", 
    "Eval file at pathString as if from the command line in it's folder.")
    
    launchFile := method(path, args,
	args ifNil(args = List clone)
	Lobby args := args
	Lobby launchPath :=  path pathComponent
	Directory setCurrentWorkingDirectory(Lobby launchPath)
	self doFile(path)
    )
    
    docSlot("println", "Same as print, but also prints a new line. Returns self.")
    println := method(self print; write("\n"); self)





    docSlot("?(aMessage)", """
    description: Sends the message aMessage to the receiver if it can respond to it. Example:
    <pre>
    MyObject test // performs test
    MyObject ?test // performs test if MyObject has a slot named test
    </pre>
    The search for the slot only follows the receivers proto chain. 
    """)

    setSlot("?", method(
	m := thisMessage argAt(0)
	if (self getSlot(m name), self doMessage(m, sender), Nil)
    ))

    docSlot("ancestors", 
    "Returns a list of all of the receiver's ancestors as found by recursively following the protos links.") 

  ancestors := method(a, 
	if(a, if(a contains(self), return a), a = List clone) 
	a append(self)
	self protos foreach(p, p ancestors(a))
	a
  )
    
    docSlot("super(aMessage)", 
    """Sends the message aMessage to the receiver's proto with the context of self. Example:
    <pre>
    self test(1, 2)   // performs test(1, 2) on self
    super(test(1, 2)) // performs test(1, 2) on self proto but with the context of self
    </pre>
    """)

  setSlot("super", method(
	senderSlotContext := sender getSlot("slotContext")
	m := thisMessage argAt(0)
	m ifNil(Exception raise("Object", "super requires an argument"))
	senderSlotContext ifNil(Exception raise("Object", "super called outside of block context"))
	slotName := m name
	ancestor := senderSlotContext ancestorWithSlot(slotName)
	if(ancestor == Nil or ancestor uniqueId == senderSlotContext uniqueId, Exception raise("Object", "super slot " .. slotName .. " not found"))
	ancestor getSlot(slotName) performOn(sender getSlot("self"), sender, m, ancestor)
))

    docSlot("resend", 
    """Send the message used to activate the current method to the Object's proto.
    For example;
    <pre>
    Dog := Mammal clone do(
	init := method(
	    resend
	)
    )
    </pre>
    calling Dog init will send an init method to Mammal, but using the Dog's context.
    """)

  setSlot("resend", method(
	senderSlotContext := sender getSlot("slotContext")
	senderSlotContext ifNil(Exception raise("Object", "resend called outside of block context"))
	m := sender thisMessage
	slotName := m name
	ancestor := senderSlotContext ancestorWithSlot(slotName)
	if(ancestor == Nil or ancestor uniqueId == senderSlotContext uniqueId, Exception raise("Object", "resend slot " .. slotName .. " not found"))
	ancestor getSlot(slotName) performOn(sender getSlot("self"), sender, m, ancestor)
))
  do := method(
	self appendProto(sender)
	self doMessage(thisMessage argAt(0), target)
	self removeProto(sender)
	self
  )
    
    docSlot("list(...)", "Returns a List containing the arguments.")
    list := method(thisMessage argsEvaluatedIn(sender))
    
    docSlot("..(arg)", " .. is an alias for: method(arg, self asString append(arg asString))")
    setSlot("..", method(arg, self asString appendSeq(arg asString)))

    docSlot("newSlot(slotName, aValue)", """
    description: Creates a getter and setter for the slot with the name slotName 
    and sets it's default the value aValue. Returns self. For example, 
    newSlot("foo", 1) would create slot named foo with the value 1 as well as a setter method setFoo().
    """)

    newSlot := method(name, value, self setSlot(name, value); self setSlot("set" .. name asCapitalized, doString("method(" .. name .. " = sender doMessage(thisMessage argAt(0)); self)")); value)
    
    docSlot("println", "Same as print, but also prints a new line. Returns self.")
    println := method(self print; write("\n"); self)

    docSlot("in(aList)", "Same as: aList contains(self)")
    in := method(aList, aList contains(self)) 
);
Lobby IoVM ImmutableSequence := ""
Lobby IoVM String := ImmutableSequence

Sequence oldStringSlots := Map clone do(
atPut("upper", "asUppercase")
atPut("lower", "asLowercase")
atPut("appendPath", "Path with(,)")
atPut("upper", "asUppercase")
atPut("removeFromTo", "removeSlice")
atPut("insert", "atInsertSeq(index, seq)")
atPut("fromTo", "slice")
atPut("substring", "slice")
atPut("beginsWith", "beginsWithSeq")
atPut("endsWith", "endsWithSeq")
atPut("after", "afterSeq")
atPut("before", "beforeSeq")
atPut("capitalized", "asCapitalized")
atPut("containsAnyCase", "containsAnyCaseSeq")
atPut("contains", "containsSeq")
atPut("swapIndexWithIndex", "swapIndices")
atPut("random", "anyOne")
atPut("randomize", "shuffle")
atPut("removeLastPathComponent", "pathComponent")
atPut("centered", "[no longer supported]")
atPut("leftJustified", "[no longer supported]")
atPut("rightJustified", "[no longer supported]")
atPut("length", "size")
atPut("find", "findSeq")
)

//Lobby Protos IoVM Buffer := Sequence clone
//Buffer before := method(s, i := self findSeq(s); if (i == Nil, self, slice(0, i)))
//Buffer after  := method(s, i := self findSeq(s); if (i == Nil, Nil, slice(i + s size, size)))

Sequence forward := method(
	slotName := thisMessage name
	newSlot := oldStringSlots at(slotName)
	if (newSlot, Exception raise("Sequence", "does not respond to " .. slotName .. " (this method has been replaced by " .. newSlot .. ")"))
	Exception raise("Sequence", "does not respond to " .. slotName)
)

Sequence do(
    makeFirstCharacterLowerCase := method(
	if(self size > 0, self atPut(0, self at(0) asLowerCase))
    )

    makeFirstCharacterUpperCase := method(
	if(self size > 0, self atPut(0, self at(0) asUpperCase))
    )

    slicesBetween := method(startSeq, endSeq,
	chunks := List clone
	lastIndex := 0
	while (startIndex := self findSeq(startSeq, lastIndex),
	    endIndex := self findSeq(endSeq, startIndex + startSeq size)
	    endIndex ifNil(break)
	    chunks append(self slice(startIndex + startSeq size, endIndex))
	    lastIndex := endIndex + endSeq size
	)
	chunks
    )
)

/*
Sequence pySplit := method(sep,
    result := List clone
    last := 0
    i := 0
    while (i = findSeq(sep, i),
        result append(slice(last, i))
        last = i+1
        i = i + 1
    )
    result append(slice(last, size))
    result
)
*/
;
Date secondsToRun := method(
	t1 := Date clone now
	sender doMessage(thisMessage argAt(0))
	dt := Date clone now secondsSince(t1)
);

Directory do(
    docSlot("with(aPath)", "Returns a new instance with the provided path.")
    with := method(path, self clone setPath(path))

    parentDirectory := method(
	if (path == ".", return Nil)
	p := self path removeLastPathComponent	
	if (p == "", p = ".")
	Directory clone setPath(p)
    )

    folders := method(
	self items select(i, item, item type == "Directory" and item name beginsWithSeq(".") == Nil)
    )

    isAccessible := method(
        r := self
	try (self items) catch(Exception, e, r = Nil)
	r
    )

    parents := method(
	list := List clone
	d := self
	while(d = d parentDirectory, list append(d))
	list reverse
    )

    accessibleParents := method(
	parents select(i, p, p isAccessible)
    )
    
    files := method(
	//writebr("path = ", path)
	items select(i, v, v type == "File")
    )
    
    fileNames := method(
	files mapInPlace(i, v, v name)
    )
    
    fileNamed := method(name,
	files detect(i, v, v name == name)
	//File clone setPath(Path with(path, name))
    )

    filesWithExtension := method(ext,
      if (ext containsSeq(".") == Nil, ext = "." .. ext)
      files select(f, f name endsWithSeq(ext))
    )

    folderNamed := method(name,
	folders detect(i, v, v name == name)
    )

    remove := method(
        File clone setPath(self path) remove
    )

    folderNamedCreateIfAbsent := method(name,
	f := folderNamed(name)
	if (f, return f)
	createSubdirectory(name)
	folderNamed(name)
    )

    createFileNamed := method(name,
	f := fileNamed(name)
	if (f, return f)
	return File clone setPath(Path with(path, name))
    )
)


;

File do(
	docSlot("asSeq", "Returns the contents of the file as a mutable Sequence.")
	asSeq := method(self asString asMutable)

	docSlot("with(aPath)", "Returns a new instance with the provided path.")
	with := method(path, self clone setPath(path))
)


;

List do(
    docSlot("sum", "Returns the sum of the items.")
    sum := method(s := 0; self foreach(v, s = s + v); s)

    docSlot("average", "Returns the average of the items.")
    average := method(self sum / self size)

    docSlot("shuffle", "Randomizes the order of the elements in the receiver. Returns self.")
    shuffle := method(for(i, 1, size - 1, swapIndices(i, Random value(i, size) floor)))

    docSlot("anyOne", "Returns a random element of the receiver or Nil if the receiver is empty.")
    anyOne := method(at(Random value(0, size) floor))
 
    docSlot("removeFirst", "Returns the first item and removes it from the list. Nil is returned if the list is empty.")
    removeFirst := method(if(self size != 0, removeAt(0)))

    docSlot("removeLast", "Returns the last item and removes it from the list. Nil is returned if the list is empty.")
    removeFirst := method(self pop)

    docSlot("join(optionalString)", "Returns a Sequence of the concatenated items with optionalString between each item or simply the concatenation of the items if no optionalString is supplied.")
    join := method(arg,
	s := Sequence clone
	if (arg) then (
	    max := self size - 1
	    self foreach(i, v, s appendSeq(v); if(i != max, s appendSeq(arg)))
	) else (
	    self foreach(v, s appendSeq(v))
	)
	s
    )

    docSlot("insertAfter(item, afterItem)", "Inserts item after first occurance of afterItem and returns self. If afterItem is not found, item is appended to the end of the list.")
    insertAfter := method(item, afterItem,
       i := self indexOf(afterItem) 
       if(i, self atInsert(i + 1, item), self append(item))
       self
    )

    docSlot("insertBefore(item, beforeItem)", "Inserts item before first occurance of beforeItem or to the end of the list if beforeItem is not found. Returns self.")
    insertBefore := method(item, beforeItem,
       i := self indexOf(beforeItem)
       if(i, self atInsert(i, item), self append(item))
       self
    )

    docSlot("insertAt(item, index)", "Inserts item at the specified index. Raises an exception if the index is out of bounds. Returns self.")
    insertAt := method(item, index, self atInsert(index, item))
);
Number constants := Object clone do(
    docSlot("constants pi", "Returns the constant pi.")
    pi := 3.14159265358979323846264338327950288

    docSlot("constants NaN", "Returns a Not a Number constant.")    
    NaN := (1/0)
    
    docSlot("constants e", "Returns the constant e.")
    e  := 2.71828182845904523536028747135266249
);
CGI := Object clone do(

docCopyright("Steve Dekorte", 2004)
docLicense("BSD revised")
docDescription("""
CGI supports accessing CGI parameters passed in environment variable or standard input by a web servers like Apache.Example use;
<pre>
#!./ioServer

write("Content-type:text/html\n\n")
write("&lt;!doctype html public \"-//W3C/DTD HTML 4.0/EN\"&gt;\n")
write("Got the following CGI parameters:&lt;PRE&gt;")

form = CGI clone parse

if(form) then (
  form foreach(k, v, write("  ", k, " = ", v, "&lt;BR&gt;"))
) else ( 
  write("no form variables found\n") 
)</pre>""")

    docSlot("parse", "Parses the QUERY_STRING environment variable (or standard input if there is no QUERY_STRING environment variable) and returns a Map containing key/value query value pairs.")

    parse := method(
      q := System getenv("QUERY_STRING")
      contentLength := System getenv("CONTENT_LENGTH")
      
      if (contentLength, 
	post := File clone standardInput readStringOfLength(contentLength asNumber)
	q = if (q != "", q .. "&" .. post, post)
      )
      //write("<PRE> q := ", q, "</PRE>\n")
      return self parseString(q)
    )

    parseString := method(q, 
      q = q asMutable replaceSeq("+", " ")

      form := Map clone
      q split("&") foreach(i, v, 
	kv := v split("=")
	if(kv size == 2) then (
	  k := kv at(0)
	  v := decodeUrlParam(kv at(1))

	  if (form hasKey(k), 
	    vallist := List clone

	    // this already exists, so we want to take the value of k, 
	    // and make a list of the values, 
	    // remove the original k/v then add k/v as list back.

	    if (form at(k) type == "List", 
		    //it's a list already, so just add the value at the end
		    vallist = form at(k)
		    vallist add(v)
	    ) else (
		    vallist add(form at(k))  
		    vallist add(v)
	    )

	    form removeAt(k)
	    form atPut(k, vallist)
	  )  else (
	    form atPut(k, v)
	  )
	) else ( 
	  kv := kv at(0) split(",")
	  form atPut("imageMapX", kv at(0))
	  form atPut("imageMapY", kv at(1))
	)
      )
      return form
    )

    urlCodes := Map clone

    setup := method(
      for (i, 0, 255,
	code := i asString toBase(16) asUppercase
	if (code size == 1, code = "0" .. code)
	/*write(i, " := ", i asCharacter, " := ", "%" .. code, "<BR>")*/
	urlCodes atPut(i asCharacter, "%" .. code) 
      )
    )

    setup

    urlPlains := Map clone do(
	    atPut(" ", "+")
	    atPut("*", "*")
	    atPut("-", "-")
	    atPut(".", ".")
	    for (i, 48, 57, atPut(i asCharacter, i asCharacter))
	    for (i, 65, 90, atPut(i asCharacter, i asCharacter))
	    atPut("_", "_")
	    for (i, 97, 122, atPut(i asCharacter, i asCharacter))
    )

    docSlot("encodeUrlParam(aString)", "Returns a URL encoded version of aString.")

    encodeUrlParam := method(sIn,
	    sOut := Sequence clone
	    sIn foreach(i, v,
		    c := v asCharacter
		    p := urlPlains at(c)
		    sOut appendSeq(if(p, p, urlCodes at(c)))
	    )
	    return sOut asString
    )

    docSlot("decodeUrlParam(aString)", "Returns a URL decoded version of aString.")

    decodeUrlParam := method(s,
      urlCodes foreach(k, v, s = s replaceSeq(v, k))
      return s
    )

)
;

Sandbox do(
    docSlot("printCallback(string)", 
     "default implementation is; method(string, string print)")
    printCallback := method(string, string print)
);
docSlot("getOptions(args)", """
	This primitive is used to get command line options similar to Cs getopt(). 
	It returns a map in containing the left side of the argument, with the 
	value of the right side. (The key will not contain
	the beginning dashes (--).
 	<p>
	Example:
 	<pre>
	options := System getOptions(args)
	options foreach(k, v,
	  if(v type == List type,
	    v foreach(i, j, writeln("Got unnamed argument with value: " .. j))
	    continue
	  )
	  writeln("Got option: " .. k .. " with value: " .. v)
	)
 	</pre>
""")


System do(
  getOptions := method(arguments,
    opts := Map clone
    optname := Sequence clone
    optvalue := Sequence clone
    optsNoKey := List clone
  
    arguments foreach(i, arg,
      if(arg beginsWithSeq("--") isNil) then(
        optsNoKey append(arg)
        continue
      )
      if(arg containsSeq("=")) then(
        optname = arg clone asMutable clipAfterStartOfSeq("=") clipBeforeEndOfSeq("--")
        optvalue = arg clone asMutable clipBeforeEndOfSeq("=")
      ) else(
        optname = arg clone asMutable clipBeforeEndOfSeq("--")
        optvalue = ""
      )
      opts atPut(optname, optvalue)
    )
    if(optsNoKey last != Nil, opts atPut("", optsNoKey))
    opts
  )
)

;
Path := Object clone do(
	with := method(
		s := Sequence clone
		thisMessage arguments foreach(arg,
			s appendPathSeq(sender doMessage(arg))
		)
		s asSymbol
	)
)

;
Importer := Object clone do(
    docDescription("A simple search path based auto-importer.")

    paths := List clone append("")
    debug := Nil
    addSearchPath := method(p, paths append(p))
    extensions := list("io")

    executeString := method(name, s,
	Lobby doString(s, name)
	r := Lobby getSlot(name)
	if(r, return r)
	Exception raise("Importer", "slot '" .. name .. "' missing after file load")
	Nil    
    )
    
    execute_io := method(path, name, extension,
	p := Path with(path, name .. "." .. extension)
	s := File clone setPath(p) asBuffer
	executeString(name, s)
    )

    find := method(name, theObject,
	p := ?launchPath
	if(p and paths contains(p) == Nil, paths atInsert(0, p))  
	if(name at(0) isUpperCase, paths foreach(p, 
	    extensions foreach(extension,
		fullPath := Path with(p, name .. "." .. extension)
		if(debug, write("Importer search:", p, "\n"))
		if(File clone setPath(fullPath) exists, 
		    r := self perform("execute_" .. extension, p, name, extension)
		    if (r, return r)
		)
	    )
	  )
	)
	Exception raise("Importer", theObject type .. " does not respond to '" .. name .. "'")
    )

    docSlot("turnOn", "Turns on the Importer. Returns self.")    
    turnOn := method(Lobby forward := method(Importer find(thisMessage name, self)); self)

    docSlot("turnOff", "Turns off the Importer. Returns self.")    
    turnOff := method(Lobby removeSlot("forward"); self)
    turnOn
);
