// Tests by Mike Austin

failures := List clone

indentLevel := -1

indent := method(
  //for(i, 0, indentLevel, write(indentLevel, "    "))
  i := 0
  while (i < indentLevel, write("    "); i = i + 1)
)

beginSection := method(name,
  indentLevel = indentLevel + 1
  write("\n")
  indent
  write(name, "\n")
  indent
  for (i, 0, name size -1, write("-"))
  write("\n\n")
)

endSection := method(
  indentLevel = indentLevel - 1
)

beginSection("Io primitive tests")

assert := block(str,
  indent
  result := Lobby doString(str)
  write(if(result, "OK   ", "FAIL "))
  write(str, "\n")
  if(result == Nil) then (
    failures add(str)
 )
)

dprint := block(str,
  indent
  write("     ", str, " ")
  o := Lobby doString(str)

/*
  if(o getSlot("print") == Object getSlot("print"),
    write("(Object)\n"), write("(", o, ")\n")
 )
*/
  write("\n")
)

beginSection("Numbers")

    beginSection("Equality")
      dprint("a := 10 ")
      dprint("b := a  ")
      assert("a == 10")
      assert("a == b ")
    endSection

    beginSection("Precision")
      assert("2 * 1.123456789123456789 == 2.2469135782469135")
      assert("2 * 1.123456789123456789 != 2.246913578246913")
      assert("10.49 roundUp == 10")
      assert("10.50 roundUp == 11")
      assert("10.51 roundUp == 11")
      assert("10.49 roundDown == 10")
      assert("10.50 roundDown == 10")
      assert("10.51 roundDown == 11")
    endSection

    beginSection("Precedence")
      assert("2 + 3 * 4 - 5 * 6 / 7 == 2 + (3 * 4) - (5 * 6) / 7")
    endSection

    beginSection("Functions")
      dprint("n := 1000 sqrt asString(0, 10)")
      // 31.62277660168379
      assert("1000 sqrt >= 31.6227766")
      assert("1000 sqrt < 31.62277661")
      dprint("n := -10")
      assert("n abs == 10")
      assert("-10 abs == 10")
      assert("10 between(9.9999, 10.0001)")
      assert("10 sin == -0.5440211108893698")
      assert("10 cos == -0.8390715290764524")
    endSection

    beginSection("Direct assignment")
      dprint("a := 5")
      assert("a = 10; a == 10")
      assert("a < -10 == Nil")
      assert("a<(-10) == Nil")
    endSection

    beginSection("Bitwise operations")
      assert("2 shiftLeft(1) == 4")
      assert("2 shiftRight(1) == 1")
      //assert("3 | (4) == 7")
      assert("3 bitwiseOr(4) == 7")
      //assert("3 & (4) == 0")
      assert("3 bitwiseAnd(4) == 0")
    endSection

    beginSection("Characters")
      assert("65 isLetter > 0")
      assert("30 isLetter == Nil")
      assert("47 isDigit == Nil")
      assert("48 isDigit > 0")
      assert("57 isDigit > 0")
      assert("58 isDigit == Nil")
    endSection
  endSection

  beginSection("Dates")

    dprint("d := Date clone now")
    assert(""" d asString("%Y") asNumber == d year """)
    assert(""" d asString("%m") asNumber == d month """)
    assert(""" d asString("%d") asNumber == d day """)
    assert(""" d asString("%H") asNumber == d hour """)
    assert(""" d asString("%M") asNumber == d minute """)
    assert(""" d asString("%S") asNumber == d second floor """)
    assert(""" d asString("%Z") == d zone """)

  endSection

  beginSection("Strings")

    beginSection("Split")
      assert(""" "abcxxx123xxxdef" split("xxx") at(1) == "123" """)
    endSection

    beginSection("Size")
      dprint("str := \"test\"")
      assert("str size == 4")
    endSection

    beginSection("Append")
      dprint("b := str .. 1")
      dprint("c := str .. \"123456789\"")
      assert("str size == 4")
      assert("b size == 5")
    endSection


    beginSection("At")
      assert("b == \"test1\"")
      assert("c == \"test123456789\"")
      assert("str at(0) == 116")
      assert("str at(1) == 101")
      assert("str at(-3) == 101")
      assert("str at(4) == 0")
    endSection

    beginSection("Conversion")
      assert("\"1.2\" asNumber == 1.2")
    endSection

    beginSection("Case")
    assert("str asCapitalized == \"Test\"")
    assert("str asUppercase == \"TEST\"")
    endSection

    beginSection("Search")
      assert("str isEqualAnyCase(\"TeSt\")")
      assert("str find(\"st\") == 2")
      assert("str contains(\"st\") != Nil")
      assert("str contains(\"xx\") == Nil")
      assert("str beginsWithSeq(\"te\") != Nil")
      assert("str endsWithSeq(\"st\") != Nil")
    endSection
  endSection


  beginSection("Objects")
    beginSection("Clone")
      dprint("a := Object clone")
      dprint("b := a clone")
      assert("a proto == Object")
      assert("b proto == a")
      write("\n")
    endSection

    /*
    beginSection("Super")
      dprint("a := Object clone")
      dprint("a x := 1")
      dprint("a foo := method(x)")
      dprint("b := a clone")
      dprint("b x := 2")
      dprint("b foo := method(super(foo))")
      assert("a foo == 1")
      assert("b foo == 2")
    endSection
    */

    beginSection("Mutable primitives")
      dprint("str := \"foobar\"")
      assert("str proto == String")

      write("\n")
      dprint("MyNumber := Number clone")
      dprint("MyNumber _test := method(1)")
      assert("MyNumber proto == Number")

      assert("10 perform(\"sin\") == -0.5440211108893698")
      assert("for(i, 1, 10, if(i == 5, break i)) == 5")
    endSection

    beginSection("Actors")
      dprint("setSchedulerSleepSeconds(0.001)")

      dprint("A := Object clone")
      dprint("A s := Buffer clone")
      dprint("A test := method(a, for(i, 1, 2, s appendSeq(a, i, \".\"); yield))")

      dprint("A clone @@test(1); A clone @@test(2)")
      dprint("wait(.1)")
      //indent; write("A s := ", A s, "\n")
      assert("A s asString == \"11.21.12.22.\"")
    endSection

    beginSection("Futures")
      dprint("obj := Object clone")
      dprint("obj double := method(v, return v * 2)")
      dprint("future := obj @double(2)")
      assert("future == 4")
    endSection
  endSection
endSection

if (failures size > 0) then (
  beginSection("Failed tests")
  failures foreach(i, o, indent; write("  ", o, "\n"))
  endSection
  write("\nNote: The precedence and abs bugs are known problems.\n\n\n")
  return -1
) else (
  write("All tests passed.\n\n")
  return 1
)


