

pi := 3.141592653589793
solar_mass := (4 * pi * pi)
days_per_year := 365.24

planet := method(x, y, z, vx, vy, vz, mass,
    p := Planet clone
    p position := vector(x, y, z)
    p velocity := vector(vx, vy, vz)
    p mass := mass
    p
)

advance := method(nbodies, bodies, dt,
    for (i, 0, nbodies,
	b = bodies at(i)
	for (j, i + 1, nbodies,
	    b2 = bodies at(j)
	    distance := b position distanceTo(b2 position)
	    mag := dt / distance cubed

	    //db := distance clone *= b2 mass *= mag
	    //b velocity -= db
	    b velocity -*= (distance, b2 mass * mag)
	    b2 velocity -*= (distance, b mass * mag)
	)
    )

    for (i, 0, nbodies,
	b = bodies at(i)
	b position +*= (dt, b velocity)
    )
)

energy := method(nbodies, bodies,
    e := 0.0
    for (i, 0, nbodies,
	b := bodies at(i)
	e = e + 0.5 * b mass * b velocity dotProduct(b velocity)
	for (j, i + 1, nbodies,
	    b2 := bodies at(j)
	    distance := b position distanceTo(b2 position)
	    e = e - (b mass * b2 mass) / distance
	)
    )
    e
)

offset_momentum := method(nbodies, bodies,
  sun := bodies at(0)
  p := vector(0, 0, 0)
  bodies foreach(b, p +*= (b velocity, b mass))
  p negate /= solar_mass
  sun velocity copy(p)
)

bodies = list(
  list(                               /* sun */
    0, 0, 0, 0, 0, 0, solar_mass
  ),
  list(                               /* jupiter */
    4.84143144246472090e+00,
    -1.16032004402742839e+00,
    -1.03622044471123109e-01,
    1.66007664274403694e-03 * days_per_year,
    7.69901118419740425e-03 * days_per_year,
    -6.90460016972063023e-05 * days_per_year,
    9.54791938424326609e-04 * solar_mass
  ),
  list(                               /* saturn */
    8.34336671824457987e+00,
    4.12479856412430479e+00,
    -4.03523417114321381e-01,
    -2.76742510726862411e-03 * days_per_year,
    4.99852801234917238e-03 * days_per_year,
    2.30417297573763929e-05 * days_per_year,
    2.85885980666130812e-04 * solar_mass
  ),
  list(                               /* uranus */
    1.28943695621391310e+01,
    -1.51111514016986312e+01,
    -2.23307578892655734e-01,
    2.96460137564761618e-03 * days_per_year,
    2.37847173959480950e-03 * days_per_year,
    -2.96589568540237556e-05 * days_per_year,
    4.36624404335156298e-05 * solar_mass
  ),
  list(                               /* neptune */
    1.53796971148509165e+01,
    -2.59193146099879641e+01,
    1.79258772950371181e-01,
    2.68067772490389322e-03 * days_per_year,
    1.62824170038242295e-03 * days_per_year,
    -9.51592254519715870e-05 * days_per_year,
    5.15138902046611451e-05 * solar_mass
  )
)

NBODIES := bodies size

n = args at(1) asNumber

offset_momentum(NBODIES, bodies)
write(energy(NBODIES, bodies) asString(0, 9))
for (i, 1, n,
    advance(NBODIES, bodies, 0.01)
)
write(energy(NBODIES, bodies) asString(0, 9))


