
Object asString := method((self type asBuffer .. " (" .. self uniqueId .. ")") asString)

AssertionFailedError := Exception clone

TestFailure := Object clone do(
    newSlot("test", Nil)
    newSlot("exception", Nil)
    description := method("In " .. test name .. " " .. exception backTraceString .. "\n\n")
)

TestResult := Object clone do(
    runCount := 0
    failureCount := 0
    errorCount := 0
    init := method(
        self testFailures := List clone
        self testErrors := List clone
    )
    addTestFailure := method(testFailure, testFailures append(testFailure))
    addTestError := method(testError, testErrors append(testError))
    testFailuresDescription := method(
        description := ""
        testFailures foreach(index, testFailure, description = description .. testFailure description)
        description
    )
    testErrorsDescription := method(
        description := ""
        testErrors foreach(index, testError, description = description .. testError description)
        description
    )
    shortSummary := method("" .. runCount .. " run, " .. errorCount .. " errors, " .. failureCount .. " failures")
    summary := method(testErrorsDescription .. "\n" .. testFailuresDescription .. "\n" .. shortSummary)
    testStarted := method(runCount = runCount + 1)
    testFailed := method(test, exception,
        testFailure := TestFailure clone setTest(test) setException(exception)
        addTestFailure(testFailure)
        failureCount = failureCount + 1
    )
    testBroke := method(test, exception,
        testError := TestFailure clone setTest(test) setException(exception)
        addTestError(testError)
        errorCount = errorCount + 1
    )
)

TestCase := Object clone do(
    testWithName := method(aName,
        test := self clone
        test name := aName
        test
    )
    setUp := method(Nop)
    tearDown := method(Nop)
    runWithResult := method(result,
        result testStarted
        try(
            setUp
            doString(name)
        ) catch(AssertionFailedError, anAssertionFailedError,
            result testFailed(self, anAssertionFailedError)
        ) catch(Exception, anException,
            result testBroke(self, anException)
        )
        tearDown
    )
    run := method(
        result := TestResult clone
        runWithResult(result)
        result
    )
    failWithMessage := method(message,
        AssertionFailedError raise("assertion failed", message)
    )
    failComparison := method(expected, actual, message,
        failWithMessage("expected '" .. expected asString .. "' but was '" .. actual asString .. "'")
    )
    fail := method(
        failWithMessage("")
    )
    assertTrue := method(aCondition,
        if(aCondition == Nil,
            failWithMessage("not true")
        )
    )
    assertFalse := method(aCondition,
        if(aCondition,
            failWithMessage("not false")
        )
    )
    assertEquals := method(expected, actual,
        if(expected != actual,
            failComparison(expected, actual)
        )
    )
    assertNotEquals := method(expected, actual,
        if(expected == actual,
            failWithMessage("equal objects")
        )
    )
    assertEqualsWithinDelta := method(expected, actual, delta,
        if(((expected - actual) abs > delta),
            failWithMessage("expected " .. expected .. " but was " .. actual .. " (allowed delta: " .. delta .. ")")
        )
    )
    assertSame := method(expected, actual,
        if(expected uniqueId != actual uniqueId,
            failComparison(expected, actual)
        )
    )
    assertNotSame := method(expected, actual,
        if(expected uniqueId == actual uniqueId,
            failWithMessage("same object")
        )
    )
    assertNil := method(nilValue,
        if(nilValue != Nil,
            failWithMessage("not Nil")
        )
    )
    assertNotNil := method(nilValue,
        if(nilValue == Nil,
            failWithMessage("Nil")
        )
    )
    assertRaisesException := method(
        try(
            sender doMessage(thisMessage argAt(0))
            failWithMessage("Should have raised Exception")
        ) catch(AssertionFailedError, failed,
            failed pass
        ) catch(Exception) pass
    )
    suite := method(
        suite := TestSuite clone
        self foreach(slotName, slotValue, if(slotName beginsWithSeq("test"), suite addTest(self testWithName(slotName))))
        suite
    )
    knownBug := method(
        write("known bug in ", self name, ": ", thisMessage argAt(0) code, "\n")
    )
)

TestSuite := Object clone do(
    init := method(self tests := List clone)
    suite := method(self)
    addTest := method(test,
        self tests append(test)
        self
    )
    addSuite := method(test,
        addTest(test suite)
    )
    addFileAtPath := method(aFileName, aPath,
        self doFile(Path with(aPath, aFileName .. ".io"))
        if(self hasSlot(aFileName .. "Suite"),
            addSuite(doMessage(Message fromString(aFileName .. "Suite")))
        ,
            addSuite(doMessage(Message fromString(aFileName)))
        )
    )
    addFile := method(aFileName,
        self addFileAtPath(aFileName, launchPath)
    )
    runWithResult := method(result,
        tests foreach(index, test, test runWithResult(result))
    )
    run := TestCase getSlot("run")
)
