/*#io
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")*
docDescription("Basic command line interace with input callback.")
*/

#include "Base.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "CLI.h"

CLI *CLI_new(void)
{
    CLI *self = (CLI *)calloc(1, sizeof(CLI));
    self->buffer = ByteArray_new();
    CLI_prompt_(self, ">");
    return self;
}

void CLI_free(CLI *self)
{
    ByteArray_free(self->buffer);
    free(self->prompt);
    free(self);
}

void CLI_prompt_(CLI *self, const char *s)
{
    self->prompt = (char *)realloc(self->prompt, strlen(s) + 1);
    strcpy(self->prompt, s);
}

void CLI_target_callback_(CLI *self, void *target, CLICallback *callback)
{
    self->target = target;
    self->callback = callback;
}

/*
 void CLI_input(CLI *self)
 {
     ByteArray_setSize_(self->buffer, 0);
     
     for (;;) 
     {    
	 int c = fgetc(stdin);
	 
	 if (c == EOF) 
	 { 
	     self->eof = 1; break; 
	 }
	 else 
	 {
	     if (c == '\n') 
	     { 
		 break; 
	     }
	     else 
	     { 
		 ByteArray_appendChar_(self->buffer, c);
	     }
	 }
     }
 }
 */

int CLI_eof(CLI *self) 
{ 
    return self->eof; 
}

void CLI_run(CLI *self) 
{
    self->done = 0;
    self->eof  = 0;
    
    while (!self->done && !self->eof)
    {
	fputs(self->prompt, stdout);
	CLI_input(self); 
	(self->callback)(self->target, ByteArray_asCString(self->buffer));
    }
}

void CLI_exit(CLI *self) 
{ 
    self->done = 1; 
}


